#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>

#define MAX 3

typedef struct lugar
{
   int posh;
   int posv;
} LUGAR;

typedef struct nave 
{
   int posH;
   int posV;
   int tipo;   
   LUGAR lugares[3];
   int aflote;
} NAVE;

int main(int argc, char *argv[])
{
   
   pid_t pidClienteServidor, pidClienteGnuplot, pidServidorGnuplot;
   
   int pipaClienteGnuplot[2];
   int pipaServidorGnuplot[2];

   int portNumberLocal;
   int portNumberDestino;
   char hnameLocal[64];
   char hostDestino[64];
   char bufCliente[1024];
   char bufServidor[1024];
   char bufClienteServidor[100]; 
   char mensajeCliente[100];
   char mensajeServidor[100];
   char *mensaje;
   char bufPipaCliente[100];
   char bufPipaServidor[100];
   int socketCliente, socketServidor, conexionServidor, largoServidor, largoCliente, socketClienteServidorLocal, largoClienteServidor, conexionClienteServidor;   
   struct sockaddr_in nombreCliente, nombreServidor, nombreClienteServidor;
   struct hostent *hostServidor, *hostClienteServidor;
   int nCarCliente, nCarServidor, nCS;
   int turno;
   int token;
   int ok;
   int parte;
   char poshServidor;
   char posvServidor;
   char resultado[10];
   char nave[15];
   char accion[10];
   char tipoNave[15];
   
   int i, j;
   int posh, posv, tipo, tiroh, tirov;
   int numeroBarcos=0;
   int estado,comparacion;
   int tipoBarco;
   
   NAVE barco[MAX];
   char *embarcacion[3];
   int tablero[10][10];
   FILE *servidorGnu, *clienteGnu, *archivo;
   int portLocal;
   
   /*Revisin de los argumentos de entrada*/
   
   if (argc<7)
     {
        printf("\nbnv2: Faltan argumentos\n");
        printf("\nSintaxis: \n");
        printf("\nbnv2 <archivo_porta-avion> <archivo_fragata> <archivo_submarino> <puerto local> <host destino> <puerto destino>\n\n");
        exit(1);
     }
   
   /*Chequeo de la existencia de los archivos pasados como argumentos*/
   
   for (i=1; i<4;i++)
     {
	archivo = fopen(argv[i], "r");
	if (archivo!=NULL)
	  fclose(archivo);
	else
	  {
	     printf("\nbnv2: Faltan argumentos,  no existe el archivo %s\n\n",argv[i]);
	     exit(1);
	  }
     }
   
   for(i=1;i<4;i++)
     embarcacion[i-1]=argv[i];

   /*
    * Creacin de los parmetros de conexin a travs de sockets
    */
   
   portNumberLocal = atoi(argv[4]);
   strcpy(hostDestino,argv[5]);
   portNumberDestino = atoi(argv[6]);

   /*
    * Creacin del nmero de puerto local que utilizaran servidor y cliente locales para comunicarse
    */
   
   portLocal = 3*getpid(); 
   
    /*
     * Creacion del proceso Cliente - Servidor
     */
   
    if ((pidClienteServidor = fork()) < 0) 
    {
        perror("Error en la creacin del proceso Cliente \n\n");
        exit(1);
    }

    /*
     * Cdigo del cliente (hijo)
     */
   
   if (pidClienteServidor == 0)
     {  
	/*
	 * Creacin de la pipa para comunicarse con el tablero del cliente (gnuplot)
	 */
		
	if (pipe(pipaClienteGnuplot) < 0) 
	  {
	     perror("Error en la creacin de la pipa cliente - gnuplot\n");
	     exit(1);
	  }

	/*
	 * Se crea el proceso  que ejecuta "gnuplot" para el tablero de disparos.
	 */
	
	//printf("\nDEP:CLIENTE: Se ha iniciado el proceso cliente... \n");
	
	if ((pidClienteGnuplot = fork()) < 0)
	  {
	     perror("Error en la creacin del proceso Gnuplot (cliente)\n\n");
	     exit(1);
	  }
	
	if (pidClienteGnuplot == 0) 
	  {
	     /*
	      * Se conecta la entrada estndard al proceso cliente-gnuplot hijo para leer desde la pipa
	      */
	     dup2(pipaClienteGnuplot[0], 0);
	     close(pipaClienteGnuplot[1]);  /* Se cierra el extremo de la pipa para escritura */

	     execlp("gnuplot", "gnuplot", 0);
	     perror("Error en la ejecucin de Gnuplot (cliente)\n\n");
	     _exit(127);
	  }
	
	/*
	 * El proceso cliente no recibe datos del Gnuplot a travs de la pipa.
	 */
	close(pipaClienteGnuplot[0]);

	clienteGnu = fdopen(pipaClienteGnuplot[1],"w");  //el cliente abre la pipa para enviar datos al Gnuplot


	fprintf(clienteGnu,"load \"configuracion.gnu\"\n"); //Se cargan los comandos de configuracion necesarios
	fflush (clienteGnu);
	fprintf(clienteGnu, "plot -2\n"); //muestra el campo de batalla
	fprintf(clienteGnu, "set parametric\npause -1\n");	
	fflush (clienteGnu);

	
	/*
	 * Se espera que el servidor local cree su tablero, coloque los barcos y este listo para aceptar conexiones
	 */
	
	//printf("\nDEP.CLIENTE: a la espera de mensaje para iniciar conexiones\n"); 
	
	/*
	 * Creacin del socket del cliente para comunicarse con el serrvidor local. En esta conexin el cliente acta como servidor  
	 */
   
	socketClienteServidorLocal = socket(AF_INET, SOCK_STREAM,0);   
   
	/*
	 * Creacin de la direccin correspondiente
	 */
   
	nombreClienteServidor.sin_family = AF_INET;
	nombreClienteServidor.sin_port = htons(portLocal);
	nombreClienteServidor.sin_addr.s_addr = htonl(INADDR_ANY);
	largoClienteServidor = sizeof(struct sockaddr_in);

	/*
	 * Se une el socket a la direccin creada
	 */
   
	bind(socketClienteServidorLocal,(struct sockaddr *) &nombreClienteServidor, largoClienteServidor);

	//printf("\nDEP.CLIENTE: no hay problemas aparentes con la creacin del socket y la direccin creada para comunicarse localmente... \n"); 
   
	/*
	 * escuchando por conexiones
	 */
   
	listen(socketClienteServidorLocal,5);
   
	/*
	 * Aceptando una conexion
	 */
   
	conexionClienteServidor = accept(socketClienteServidorLocal, (struct sockaddr *) &nombreClienteServidor, &largoClienteServidor);

	//printf("\nDEP.CLIENTE: conexion local establecida! \n");
	
	while(strncmp(bufClienteServidor,"listo",5)!=0)
	  {
	     nCS = recv(conexionClienteServidor,bufClienteServidor,sizeof(bufClienteServidor),0);
	     
	  //   printf("\nDEP:CLIENTE: bufClienteServidor vale : %s",bufClienteServidor); 
	  }
	
	//printf("\nDEP.CLIENTE: se recibi mensaje para iniciar conexiones : %s\n",bufClienteServidor);
	
	/*
	 * se busca el servidor contrario en la red
	 */
	
	hostServidor = gethostbyname(hostDestino);
	
	/*
	 * Se crea el socket del cliente
	 */
	
	socketCliente = socket(AF_INET, SOCK_STREAM, 0);
	
	/*
	 * Se crea la direccin del servidor remoto al que se va a conectar
	 */
	
	nombreCliente.sin_family = AF_INET;
	nombreCliente.sin_port = htons(portNumberDestino);
	memcpy(&nombreCliente.sin_addr, hostServidor->h_addr_list[0], hostServidor->h_length);
	largoCliente = sizeof(struct sockaddr_in);
	
	/*
	 * Se intenta conectar con el servidor remoto hasta que resulte...
	 */
	
	while(connect(socketCliente,(struct sockaddr *) &nombreCliente, largoCliente)!=0);
	
	printf("Conectado con el adversario en %s \n\n",hostDestino);

	/*
	 * Se recibe el turno generado por servidor remoto
	 */
	
	nCarCliente = recv(socketCliente,bufCliente,sizeof(bufCliente),0);
	
	//printf("\nDEP.CLIENTE: turno remoto recibido...\n");
	
	/*
	 * Se recibe el turno generado por el servidor local
	 */
	
	nCS = recv(conexionClienteServidor,bufClienteServidor,sizeof(bufClienteServidor),0);
	
	//printf("\nDEP.CLIENTE:turno local recibido... \n");
	
	comparacion = strcmp(bufCliente,bufClienteServidor); 
	 
		
	if (comparacion<0)
	  {
	     /*
	      * Dispara este cliente primero
	      */
	     printf("\n\nA continuacin comenzar a disparar. \n Si desea terminar ingrese una coordenada fuera de rango.\n\n");
	     token = 1;
	  }
	else
	  {
	     printf("\n\nEsperando disparo del enemigo... \n\n");
	     token =0;
	  }
	
	while(1)
	  {
	     if (token)
	       {
		  ok=1;
		  while(ok)
		      {
			 printf("Ingrese posicin horizontal de tiro (0-9):");
			 scanf("%d",&tiroh);
			 printf("Ingrese posicin vertical de tiro (0-9):");
			 scanf("%d",&tirov);
			 if ((tiroh>=10)||(tirov>=10)||(tiroh<0)||(tirov<0)) 
			   ok=1;
			 else
			   ok=0;
		      }
		  	 
		  sprintf(bufCliente,"Disparo:%d:%d",tiroh,tirov);
		  while(send(socketCliente,bufCliente,sizeof(bufCliente),0)<0);
		  nCarCliente=recv(socketCliente,bufCliente,sizeof(bufCliente),0);
		  strcpy(mensajeCliente,bufCliente);
		  parte=0;
		  i=0;
		  mensaje=mensajeCliente;
		  while((*mensaje)!='\0')
		    {
		       switch(parte)
			 {
			  case 0:
			    if((*mensaje)!=':')
			      {
				 resultado[i]=(*mensaje);
				 mensaje++;
				 i++;
			      }
			    else
			      {
				 resultado[i]='\0';
				 mensaje++;
				 i=0;
				 parte++;
			      }
			    break;
			  case 1:
			    if((*mensaje)!=':')
			      {
				 nave[i]=(*mensaje);
				 mensaje++;
				 i++;
			      }
			    else
			      {
				 nave[i]='\0';
				 mensaje++;
				 i=0;
				 parte++;
			      }
			    break;
			  case 2:
			    if((*mensaje)!=':')
			      {
				 poshServidor=(*mensaje);
				 mensaje++;
				 i++;
			      }
			    else
			      {
				 mensaje++;
				 i=0;
				 parte++;
			      }
			    break;
			  case 3:
			    if((*mensaje)!=':')
			      {
				 posvServidor=(*mensaje);
				 mensaje++;
				 i++;
			      }
			    else
			      {
				 mensaje++;
				 i=0;
				 parte++;
			      }
			    break;
			  default:
			    break;
			 }
		    }
		  
		  if(strcmp(resultado,"Blanco")==0)
		    {
		       fprintf(clienteGnu,"\n");
		       fprintf(clienteGnu,"plot 30*sin(t)+50+100*%i,30*cos(t)+50+100*%i with lines linetype 1 lw 25 \n",tiroh,tirov); //dibuja circulo
		       fprintf(clienteGnu, "\n"); 
		       fflush(clienteGnu);
		       printf("\n==> Has impactado un %s, en la posicin (%d,%d) !!\n\n",nave,tiroh,tirov); 
		    }
		  if(strcmp(resultado,"Agua")==0)
		    {
		       fprintf(clienteGnu,"\n");
		       fprintf(clienteGnu,"plot 30*sin(t)+50+100*%i,30*cos(t)+50+100*%i with lines linetype 3 lw 25 \n",tiroh,tirov); //dibuja circulo
		       fprintf(clienteGnu, "\n"); 
		       fflush(clienteGnu);
		       printf("\n==> Al agua!!, en la posicin (%d,%d) \n\n",tiroh,tirov); 
		    }
		  sprintf(bufCliente,"token");
		  while(send(socketCliente,bufCliente,sizeof(bufCliente),0)<0);
		  token=0;
		  nCarCliente = recv(socketCliente,bufCliente,sizeof(bufCliente),0);

		  if(strncmp(bufCliente,"ganaste",7)==0)
		    {
		       /*
			* Ultimo tiro, ultimo barco.. ha ganado este cliente
			*/
		       printf("\n\n<=> Ud. ha ganado...!!!!!!     :) \n");
		       
		       /*
			* Finaliza el cliente, en caso de haber ganado
			*/
		       
		       shutdown(socketCliente,2);
		       shutdown(conexionClienteServidor,2);
		       shutdown(socketClienteServidorLocal,2);
		       fflush(stdin);
		       sleep(10);
		       fprintf(clienteGnu,"\n exit\n");
		       fflush(clienteGnu);
		       fclose(clienteGnu);
		       waitpid(pidClienteGnuplot,&estado,0);
		       exit(0);
		    }
		  }
	     else
	       {
		  /*
		   * Recuperar token enviado por el servidor remoto
		   */
		  nCS =recv(conexionClienteServidor,bufClienteServidor,sizeof(bufClienteServidor),0);
		  
		  if(strncmp(bufClienteServidor,"fin",3)==0)
		    {
		       sprintf(bufCliente,"Fin:::");
		       while(send(socketCliente,bufCliente,sizeof(bufCliente),0)<0);
		       
		       /*
			* Finaliza el cliente, en caso de haber perdido
			*/
		       
		       shutdown(socketCliente,2);
		       shutdown(conexionClienteServidor,2);
		       shutdown(socketClienteServidorLocal,2);
		       fflush(stdin);
		       sleep(10);
		       fprintf(clienteGnu,"\n exit\n");
		       fflush(clienteGnu);
		       fclose(clienteGnu);
		       waitpid(pidClienteGnuplot,&estado,0);
		       exit(0);
		    }
		  
		  /*
		   * Se sigue jugando
		   */
		  
		  if(strncmp(bufClienteServidor,"token",5)==0)
		    token=1;
		  else
		    token=0;
	       }
	  }
			      
	/*  
	 * Aqu termina el cdigo del cliente
	 */	
     }
   else
     {
	/*
	 * Cdigo correspondiente al servidor
	 */ 
	
	/*
	 * Creacin de la pipa para comunicarse con el tablero del servidor (gnuplot)
	 */ 
	
	if (pipe(pipaServidorGnuplot) < 0) 
	  {
	     perror("Error en la creacin de la pipa servidor - gnuplot\n");
	     exit(1);
	  }
	
	/*
	 * Se crea el proceso  que ejecuta "gnuplot" para el tablero de barcos.
	 */
	if ((pidServidorGnuplot = fork()) < 0)
	  {
	     perror("Error en la creacin del proceso Gnuplot (servidor)\n\n");
	     exit(1);
	  }
	
	if (pidServidorGnuplot == 0) 
	  {
	     /*
	      * Se conecta la entrada estndard al proceso servidor-gnuplot hijo para leer desde la pipa
	      */
	     dup2(pipaServidorGnuplot[0], 0);
	     close(pipaServidorGnuplot[1]);  /* Se cierra el extremo de la pipa para escritura */
	     
	     execlp("gnuplot", "gnuplot", 0);
	     perror("Error en la ejecucin de Gnuplot (servidor)\n\n");
	     _exit(127);
	  }
	
	/*
	 * El proceso servidor no recibe datos del Gnuplot a travs de la pipa.
	 */
	close(pipaServidorGnuplot[0]);
	
	servidorGnu = fdopen(pipaServidorGnuplot[1],"w");  //el servidor abre la pipa para enviar datos al Gnuplot
	
	fprintf(servidorGnu,"load \"configuracion.gnu\"\n"); //Se cargan los comandos de configuracion necesarios
	fflush (servidorGnu);
	fprintf(servidorGnu, "plot -2\n"); //muestra el campo de batalla
	fprintf(servidorGnu, "set parametric\npause -1\n\n");	
	fflush (servidorGnu);
	
	system("clear");
	
	printf("\n");
	printf("**********************************************\n");
	printf("*  Batalla Naval versin 2.0 (peer-to-peer)  *\n");	
	printf("**********************************************\n");
	printf("\n\n");
	printf("A continuacin ingresar las posiciones de las naves en el tablero. \n");
	printf("porta-aviones .......... tipo 0\n");
	printf("crucero (fragata) ...... tipo 1\n");
	printf("submarino .............. tipo 2\n");
	printf("\n\n");
	
	/* Inicializacin de ubicacin de los barcos y del tablero*/
	
	for (i=0;i<MAX;i++)
	  {
	     barco[i].posH=-1;
	     barco[i].posV=-1;
	     barco[i].tipo=-1;
	     for(j=0;j<3;j++)
	       {
		  barco[i].lugares[j].posh=-1;
		  barco[i].lugares[j].posv=-1;
	       }
	     barco[i].aflote=-1;
	  }
	
	for(i=0;i<10;i++)
	  for(j=0;j<10;j++)
	    tablero[i][j]=0;
	
	/*
	 * Ubicacin de los barcos
	 */
	
	i=0;
	while(i<MAX)
	  {
	     printf("Ubicacin de la nave tipo %d\n",i);
	     printf("\tingrese posicin horizontal (0-9):");
	     scanf("%d",&posh);
	     printf("\tingrese posicin vertical (0-9):");
	     scanf("%d",&posv);
	     
	     if ((posh<0)||(posh>9)||(posv<0)||(posh>9)) 
	       {
		  printf("\nPosicin invlida \n");
	       }
	     else
	       {
		  switch(i)
		    {
		       /*
			* Porta-aviones
			*/ 
		     case 0: 
		       switch(posh)
			 {
			  case 8:
			    if (tablero[posh-1][posv] || tablero[posh][posv] || tablero[posh+1][posv])
			      {
				 printf("\nExiste una nave en la posicin  se traslapa con alguna cercana... \n");
			      }
			    else
			      {
				 barco[i].posH=posh;
				 barco[i].posV=posv;
				 barco[i].tipo=i;
				 barco[i].aflote=3;	
				 tablero[posh][posv]=1;
				 barco[i].lugares[0].posh=posh-1;
				 barco[i].lugares[0].posv=posv;
				 barco[i].lugares[1].posh=posh;
				 barco[i].lugares[1].posv=posv;
				 barco[i].lugares[2].posh=posh+1;
				 barco[i].lugares[2].posv=posv;
				 tablero[posh-1][posv]=1;
				 tablero[posh+1][posv]=1;
				 fprintf(servidorGnu, "\nplot \"%s\" using ($1+%i-100):($2+%i) with lines 1\n", embarcacion[i], posh*100, posv*100);
				 fprintf(servidorGnu, "\n"); 
				 fflush (servidorGnu);
				 numeroBarcos++;  
				 i++;
			      }
			    break;
			  case 9:
			    if (tablero[posh-2][posv] || tablero[posh-1][posv] || tablero[posh][posv])
			      {
				 printf("\nExiste una nave en la posicin  se traslapa con alguna cercana... \n");
			      }
			    else
			      {
				 barco[i].posH=posh;
				 barco[i].posV=posv;
				 barco[i].tipo=i;
				 barco[i].aflote=3;	
				 tablero[posh][posv]=1;
				 barco[i].lugares[0].posh=posh-2;
				 barco[i].lugares[0].posv=posv;
				 barco[i].lugares[1].posh=posh-1;
				 barco[i].lugares[1].posv=posv;
				 barco[i].lugares[2].posh=posh;
				 barco[i].lugares[2].posv=posv;
				 tablero[posh-2][posv]=1;
				 tablero[posh-1][posv]=1;
				 fprintf(servidorGnu, "\nplot \"%s\" using ($1+%i-200):($2+%i) with lines 1\n", embarcacion[i], posh*100, posv*100);
				 fprintf(servidorGnu, "\n"); 
				 fflush (servidorGnu);
				 numeroBarcos++;
				 i++;
			      }
			    break;
			  default:
			    if (tablero[posh][posv] || tablero[posh+1][posv] || tablero[posh+2][posv])
			      {
				 printf("\nExiste una nave en la posicin  se traslapa con alguna cercana... \n");
			      }
			    else
			      {
				 barco[i].posH=posh;
				 barco[i].posV=posv;
				 barco[i].tipo=i;
				 barco[i].aflote=3;	
				 tablero[posh][posv]=1;
				 barco[i].lugares[0].posh=posh;
				 barco[i].lugares[0].posv=posv;
				 barco[i].lugares[1].posh=posh+1;
				 barco[i].lugares[1].posv=posv;
				 barco[i].lugares[2].posh=posh+2;
				 barco[i].lugares[2].posv=posv;
				 tablero[posh+1][posv]=1;
				 tablero[posh+2][posv]=1;
				 fprintf(servidorGnu, "\nplot \"%s\" using ($1+%i):($2+%i) with lines 1\n", embarcacion[i], posh*100, posv*100);
				 fprintf(servidorGnu, "\n"); 
				 fflush (servidorGnu);
				 numeroBarcos++;
				 i++;
			      }
			    break;
			 };
		       break;
		       /*
			* Crucero o Fragata
			*/
		     case 1:
		       switch(posh)
			 {
			  case 9:
			    if (tablero[posh-1][posv] || tablero[posh][posv])
			      {
				 printf("\nExiste una nave en la posicin  se traslapa con alguna cercana... \n");
			      }
			    else
			      {	 
				 barco[i].posH=posh;
				 barco[i].posV=posv;
				 barco[i].tipo=i;
				 barco[i].aflote=2;	
				 tablero[posh][posv]=1;
				 barco[i].lugares[0].posh=posh-1;
				 barco[i].lugares[0].posv=posv;
				 barco[i].lugares[1].posh=posh;
				 barco[i].lugares[1].posv=posv;
				 tablero[posh-1][posv]=1;
				 fprintf(servidorGnu, "\nplot \"%s\" using ($1+%i-100):($2+%i) with lines 1\n", embarcacion[i], posh*100, posv*100);
				 fprintf(servidorGnu, "\n"); 
				 fflush (servidorGnu);
				 numeroBarcos++;
				 i++;
			      }
			    break;
			  default:
			    if (tablero[posh][posv] || tablero[posh+1][posv])
			      {
				 printf("\nExiste una nave en la posicin  se traslapa con alguna cercana... \n");
			      }
			    else
			      {	 
				 barco[i].posH=posh;
				 barco[i].posV=posv;
				 barco[i].tipo=i;
				 barco[i].aflote=2;	
				 tablero[posh][posv]=1;
				 barco[i].lugares[0].posh=posh;
				 barco[i].lugares[0].posv=posv;
				 barco[i].lugares[1].posh=posh+1;
				 barco[i].lugares[1].posv=posv;
				 tablero[posh+1][posv]=1;
				 fprintf(servidorGnu, "\nplot \"%s\" using ($1+%i):($2+%i) with lines 1\n", embarcacion[i], posh*100, posv*100);
				 fprintf(servidorGnu, "\n"); 
				 fflush (servidorGnu);
				 numeroBarcos++;
				 i++;
			      }
			    break;
			 };
		       break;
		       /*
			* Submarino
			*/ 
		     case 2:	
		       if (tablero[posh][posv])
			 {
			    printf("\nExiste una nave en la posicin  se traslapa con alguna cercana... \n");
			 }
		       else
			 {
			    barco[i].posH=posh;
			    barco[i].posV=posv;
			    barco[i].tipo=i;
			    barco[i].aflote=1;	
			    tablero[posh][posv]=1;
			    barco[i].lugares[0].posh=posh;
			    barco[i].lugares[0].posv=posv;
			    fprintf(servidorGnu, "\nplot \"%s\" using ($1+%i):($2+%i) with lines 1\n", embarcacion[i], posh*100, posv*100);
			    fprintf(servidorGnu, "\n"); 
			    fflush (servidorGnu);numeroBarcos++;
			    i++;
			 };
		       break;
		     default:	
		       break;
		    }
	       }
	  }
	
	/*
	 * Creacin del socket del servidor para aceptar conexiones para comenzar el juego
	 */
	
	socketServidor = socket(AF_INET, SOCK_STREAM,0);   
	
	/*
	 * Creacin de la direccin del servidor
	 */
	
	nombreServidor.sin_family = AF_INET;
	nombreServidor.sin_port = htons(portNumberLocal);
	nombreServidor.sin_addr.s_addr = htonl(INADDR_ANY);
	largoServidor = sizeof(struct sockaddr_in);
	
	/*
	 * Se une el socket a la direccin creada
	 */
	
	bind(socketServidor,(struct sockaddr *) &nombreServidor, largoServidor);
	
	//   printf("\nDEP:SERVIDOR: no hay problemas aparentes con la creacin del socket y la direccin creada... \n"); 
   	// 
	/*
	 * Se le avisa al cliente local que comience a intentar conectarse con la contraparte remota
	 */
	
	gethostname(hnameLocal,sizeof(hnameLocal));
	hostClienteServidor = gethostbyname(hnameLocal);
	
	/*
	 * Se crea el socket del servidor local al cliente local
	 */
	
	socketClienteServidorLocal = socket(AF_INET, SOCK_STREAM, 0);
	
	/*
	 * Se crea la direccin del servidor local al que se va a conectar
	 */
	
	nombreClienteServidor.sin_family = AF_INET;
	nombreClienteServidor.sin_port = htons(portLocal);
	memcpy(&nombreClienteServidor.sin_addr, hostClienteServidor->h_addr_list[0], hostClienteServidor->h_length);
	largoClienteServidor = sizeof(struct sockaddr_in);
	
	/*
	 * Se intenta conectar con el servidor local hasta que resulte...
	 */
	
	while(connect(socketClienteServidorLocal,(struct sockaddr *) &nombreClienteServidor, largoClienteServidor)!=0);
	
	sprintf(bufClienteServidor,"listo");
	
	while(send(socketClienteServidorLocal,bufClienteServidor,sizeof(bufClienteServidor),0)<0);
	
	
	// printf("\nDEP:SERVIDOR: mensaje enviado al cliente local por el socket local correspondiente... \n");    
	/*
	 * escuchando por conexiones
	 */
	
	listen(socketServidor,5);
	
	/*
	 * Aceptando una conexion
	 */
	
	conexionServidor = accept(socketServidor, (struct sockaddr *) &nombreServidor, &largoServidor);
	
	printf("Tiene un adversario conectado desde : %s\n\n", inet_ntoa(nombreServidor.sin_addr));
	
	/*
	 * Generacin de turno del servidor. Se enva un letra ascii minscula (97 - 122) 
	 */ 
	
	srand(getpid());
	
	turno = 97 + (int) (25.0 * rand()/(RAND_MAX+1.0));
	
	/*
	 * Se enva el turno al cliente local y al cliente remoto
	 */ 
	
	sprintf(bufClienteServidor,"%d",turno);
	while(send(socketClienteServidorLocal,bufClienteServidor,sizeof(bufClienteServidor),0)<0);
	//printf("DEP.SERVIDOR: turno localmente enviado..."); 
	while(send(conexionServidor,bufClienteServidor,sizeof(bufClienteServidor),0)<0);
	
	//printf("DEP.SERVIDOR: turno remotamente enviado...");
	
	
	/*
	 * Se leen los disparos desde el socket y se verifica si son blanco o agua.. 
	 */ 
	
	while(1)
	  {
	     nCarServidor = recv(conexionServidor,bufServidor,sizeof(bufServidor),0);
	     strcpy(mensajeServidor,bufServidor);
	     mensaje=mensajeServidor;
	     parte=0;
	     i=0;
	     while((*mensaje)!='\0')
	       {
		  switch(parte)
		    {
		     case 0:
		       if((*mensaje)!=':')
			 {
			    accion[i]=(*mensaje);
			    mensaje++;
			    i++;
			 }
		       else
			 {
			    accion[i]='\0';
			    mensaje++;
			    i=0;
			    parte++;
			 }
		       break;
		     case 1:
		       if((*mensaje)!=':')
			 {
			    poshServidor=(*mensaje);
			    mensaje++;
			    i++;
			 }
		       else
			 {
			    mensaje++;
			    i=0;
			    parte++;
			 }
		       break;
		     case 2:
		       if((*mensaje)!=':')
			 {
			    posvServidor=(*mensaje);
			    mensaje++;
			    i++;
			 }
		       else
			 {
			    mensaje++;
			    i=0;
			    parte++;
			 }
		       break;
		     default:
		       break;
		    }
	       }
	     
	     if(strcmp(accion,"Disparo")==0)
	       {
		  tiroh = poshServidor-48;  //Convertir ascii a entero.. 
		  tirov = posvServidor-48;
		  if (numeroBarcos)
		    {
		       if(tablero[tiroh][tirov])
			 {
			    tablero[tiroh][tirov]=0;
			    for(i=0;i<MAX;i++)
			      {
				 for(j=0;j<3;j++)
				   if ((barco[i].lugares[j].posh==tiroh)&&(barco[i].lugares[j].posv==tirov))
				     {
					barco[i].aflote--;
					tipoBarco = barco[i].tipo;
				     }
				 
				 if (barco[i].aflote==0)
				   {
				      printf("\n<=> PELIGRO: Ha perdido una embarcacin!\n\n");
				      barco[i].aflote=-1;
				      numeroBarcos--;
				   }
				 switch (tipoBarco)
				   {
				    case 0:
				      sprintf(tipoNave,"Porta-avion");
				      break;
				    case 1:
				      sprintf(tipoNave,"Crucero");
				      break;
				    case 2:
				      sprintf(tipoNave,"Submarino");
				      break;
				    default:
				      break;
				   }
			      }	
			    fprintf(servidorGnu, "\nplot 30*sin(t)+50+100*%i,30*cos(t)+50+100*%i with lines linetype 1 lw 25 \n",tiroh,tirov); //dibuja circulo
			    fprintf(servidorGnu, "\n"); 
			    fflush(servidorGnu);
			    sprintf(bufServidor,"Blanco:%s:%d:%d",tipoNave,tiroh,tirov);
			    while(send(conexionServidor,bufServidor,sizeof(bufServidor),0)<0);
			 }
		       else
			 { 
			    fprintf(servidorGnu, "\nplot 30*sin(t)+50+100*%i,30*cos(t)+50+100*%i with lines linetype 2 lw 25 \n",tiroh,tirov); //dibuja circulo
			    fprintf(servidorGnu, "\n"); 
			    fflush(servidorGnu);
			    sprintf(bufServidor,"Agua:%d:%d",tiroh,tirov);
			    while(send(conexionServidor,bufServidor,sizeof(bufServidor),0)<0);
			 }
		    }
		  nCarServidor = recv(conexionServidor,bufServidor,sizeof(bufServidor),0);
		  if(strncmp(bufServidor,"token",5)==0)
		    {
		       if(numeroBarcos==0)
			 {
			    /*
			     * no quedan ms barcos por hundir.. el cliente remoto gano...
			     */
			    sprintf(bufServidor,"ganaste");
			    while(send(conexionServidor,bufServidor,sizeof(bufServidor),0)<0);
			    
			    printf("\n\n<=> Ud. ha perdido...!!!!!!     :( \n");
			    
			    /*
			     * Se le avisa al cliente local que todo ha concluido
			     */ 
			    
			    sprintf(bufClienteServidor,"fin");
			    while(send(socketClienteServidorLocal,bufClienteServidor,sizeof(bufClienteServidor),0)<0);
			    
			    /*
			     * Finaliza el servidor en caso de perder
			     */
			    
			    shutdown(conexionServidor,2);
			    shutdown(socketServidor,2);
			    shutdown(socketClienteServidorLocal,2);
			    fflush(stdin);
			    sleep(10);
			    fprintf(servidorGnu,"\n exit\n");
			    fflush(servidorGnu);
			    fclose(servidorGnu);
			    waitpid(pidServidorGnuplot,&estado,0);
			    exit(0);
			 }
		       else
			 {
			    /*
			     * Se continua jugando
			     */
			    sprintf(bufClienteServidor,"token");
			    while(send(socketClienteServidorLocal,bufClienteServidor,sizeof(bufClienteServidor),0)<0);
			    sprintf(bufServidor,"continua");
			    while(send(conexionServidor,bufServidor,sizeof(bufServidor),0)<0);
			 }
		    }
	       }
	     if (strcmp(accion,"Fin")==0)
	       {
		  /*
		   * Finaliza el servidor en caso de ganar
		   */
		  
		  shutdown(conexionServidor,2);
		  shutdown(socketServidor,2);
		  shutdown(socketClienteServidorLocal,2);
		  fflush(stdin);
		  sleep(10);
		  fprintf(servidorGnu,"\n exit\n");
		  fflush(servidorGnu);
		  fclose(servidorGnu);
		  waitpid(pidServidorGnuplot,&estado,0);
		  waitpid(pidClienteServidor,&estado,0);
		  exit(0);
	       }
	  }
     }
}

	     
